/***********************************************************************
*  File: corpse.c Version 1.1                                          *
*  Usage: Corpse Saving over reboots/crashes/copyovers                 *
*                                                                      *
*  By: Michael Cunningham (Romulus) IMP of Legends of The Phoenix MUD  *
*  Permission is granted to use and modify this software as long as    *
*  credits are given in the credits command in the game.               *
*  Built for circle30bpl15                                             *
*  Revamped by Timorad												   *
*																	   *
*  All Rights Reserved, Copyright (C) 1999                             *
***********************************************************************/

/* The standard includes */
#include "conf.h"
#include "sysdep.h"
#include "structs.h"
#include "comm.h"
#include "handler.h"
#include "db.h"
#include "interpreter.h"
#include "utils.h"
#include "spells.h"
#include "diskio.h"

/* External Structures / Variables */
extern struct obj_data *object_list;
extern struct room_data *world;
struct index_data *obj_index;   /* index table for object file   */
extern room_vnum frozen_start_room;

/* External Functions */
void obj_from_room(struct obj_data * obj);
void obj_to_room(struct obj_data * object, room_rnum room);
void obj_to_obj(struct obj_data * obj, struct obj_data * obj_to);
int Obj_to_store(struct obj_data *obj, FBFILE *fp, int locate, bool corpse);
int load_objects(struct char_data *ch, FBFILE *fl, bool corpse);

/* Local Function Declerations */
void save_corpses(void);
void load_corpses(void);
int corpse_save(struct obj_data *obj, FBFILE * fp, int location, bool recurse_this_tree); 
int write_corpse_to_disk(FBFILE *fp, struct obj_data *obj, int locate);

/* Tada! THE FUNCTIONS ! Yaaa! */

int corpse_save(struct obj_data *obj, FBFILE *fl, int location, bool recurse_this_tree) 
{
  /* This function basically is responsible for taking the    */
  /* supplied obj and figuring out if it has any contents. If */
  /* it does then we write those to disk.. Ad Nasum.          */

  struct obj_data *tmp;
  int result;

  if (obj) { /* a little recursion (can be a dangerous thing:) */

    /* recurse_this_tree causes the recursion to branch only
       down the corpses content's tree and not the contents of the
       room. obj->next_content points to the rooms contents
       the first time this function is called from save_corpses
       hence we avoid going down there otherwise we will save
       the rooms contents as well as the corpses contents in the 
       corpse.save file. 
    */

    if (recurse_this_tree != FALSE)
    {
      corpse_save(obj->next_content, fl, location, recurse_this_tree);
    }
    recurse_this_tree = TRUE;
    corpse_save(obj->contains, fl, MIN(0, location) - 1, recurse_this_tree);
    result = Obj_to_store(obj, fl, location, TRUE); 

    /* readjust the wieght while we do this */
    for (tmp = obj->in_obj; tmp; tmp = tmp->in_obj)
      GET_OBJ_WEIGHT(tmp) -= GET_OBJ_WEIGHT(obj);
  
    if (!result)
      return (0);
  }
  return (TRUE);
}

void save_corpses(void)
{
  /* This is basically the mother of all the save corpse functions */
  /* You can call it from anywhere in the game with no arguments */
  /* Basically any time a corpse is manipulated in any way..either */
  /* directly or indirectly you need to call this function */

  FBFILE *fl;
  struct obj_data *i, *next;
  int location = 0;

  /* Open corpse file */
  if (!(fl = fbopen(CORPSE_FILE, FB_WRITE))) {
    log("SYSERR: Writing corpse file %s", CORPSE_FILE);
	return;
  }
  
  /* Scan the object list */
  for (i = object_list; i; i = next) {
    next = i->next;      

    /* Check if its a players corpse */
    if (OBJ_FLAGGED(i, ITEM_PC_CORPSE)) {
      /* It is, so save it to a file */
      if (!corpse_save(i, fl, location, FALSE)) {           
        log("SYSERR: A corpse didnt save for some reason");
        fbclose(fl);
        return;
	  }
	}
  }   
  /* Close the corpse file */
  fbprintf(fl, "$~\n");
  fbclose(fl);
} 

void load_corpses(void) 
{
  /* Ahh load corpses.. it was cake to write them out to a file      */
  /* it was a pain to load them back up through without a character. */
  /* Because I dont have a character I couldnt figure out how to     */
  /* put objects back into the corpse the exact way they came out..  */
  /* like objects back in their container, etc. So I just decided to */
  /* Dump it all in the corpse and let the character sort it out.    */
  /* If they dont like it, screwum. They are lucky I coded this:)    */
  /* Oh, and a bunch of this code is from Patricks XAP obj's code    */

  // Now fixed, thanks to Timorad.

  FBFILE *fl;
  
  if (!(fl = fbopen(CORPSE_FILE, FB_READ))) {
    if (errno != ENOENT) { /* if it fails, NOT because of no file */
      log("SYSERR: Reading corpse file %s", CORPSE_FILE);
	}
    return;
  }

  load_objects(NULL, fl, TRUE);
  fbclose(fl);
}

