/* ************************************************************************
*   File: spec_assign.c                                 Part of CircleMUD *
*  Usage: Functions to assign function pointers to objs/mobs/rooms        *
*                                                                         *
*  All rights reserved.  See license.doc for complete information.        *
*                                                                         *
*  Copyright (C) 1993, 94 by the Trustees of the Johns Hopkins University *
*  CircleMUD is based on DikuMUD, Copyright (C) 1990, 1991.               *
************************************************************************ */

#include "conf.h"
#include "sysdep.h"

#include "structs.h"
#include "db.h"
#include "interpreter.h"
#include "utils.h"
#include "genmob.h"


/* external globals */
extern int mini_mud;

/* external functions */
SPECIAL(dump);
SPECIAL(pet_shops);
SPECIAL(postmaster);
SPECIAL(cityguard);
SPECIAL(receptionist);
SPECIAL(cryogenicist);
SPECIAL(guild);
SPECIAL(puff);
SPECIAL(fido);
SPECIAL(janitor);
SPECIAL(mayor);
SPECIAL(snake);
SPECIAL(thief);
SPECIAL(magic_user);
SPECIAL(bank);

/* local functions */
void assign_mobiles(void);
void assign_objects(void);
void assign_rooms(void);
void ASSIGNROOM(room_vnum room, SPECIAL(fname));
void ASSIGNMOB(mob_vnum mob, SPECIAL(fname));
void ASSIGNOBJ(obj_vnum obj, SPECIAL(fname));

/* functions to perform assignments */

void ASSIGNMOB(mob_vnum mob, SPECIAL(fname))
{
  mob_rnum rnum;

  if ((rnum = real_mobile(mob)) != NOBODY) 
    mob_index[rnum].func = fname;
  else if (!mini_mud)
    log("SYSERR: Attempt to assign spec to non-existant mob #%d", mob);
}

void ASSIGNOBJ(obj_vnum obj, SPECIAL(fname))
{
  obj_rnum rnum;

  if ((rnum = real_object(obj)) != NOTHING)
    obj_index[rnum].func = fname;
  else if (!mini_mud)
    log("SYSERR: Attempt to assign spec to non-existant obj #%d", obj);
}

void ASSIGNROOM(room_vnum room, SPECIAL(fname))
{
  room_rnum rnum;

  if ((rnum = real_room(room)) != NOWHERE)
    world[rnum].func = fname;
  else if (!mini_mud)
    log("SYSERR: Attempt to assign spec to non-existant room #%d", room);
}


/* ********************************************************************
*  Assignments                                                        *
******************************************************************** */

void load_guilds(void)
{
  int i;

  for (i = 0; i <= top_of_world; i++) 
	if ((ROOM_FLAGGED(i, ROOM_GUILD_SWORD)) ||
		  (ROOM_FLAGGED(i, ROOM_GUILD_MAGE)) ||
		  (ROOM_FLAGGED(i, ROOM_GUILD_THIE)) || 
		  (ROOM_FLAGGED(i, ROOM_GUILD_CLER)))
	  ASSIGNROOM(GET_ROOM_VNUM(i), guild);
  
}

struct mob_prog_data mobprogs[] = {
	{ "None."		, NULL },
	{ "Postmaster"	, postmaster }, 
	{ "Snake"		, snake },
	{ "Thief"		, thief },
	{ "Magic-user"	, magic_user },
	{ "Fido"		, fido },
	{ "Janitor"		, janitor },
	{ "Cityguard"	, cityguard},
	{ "\n"			, NULL }
};

void assign_mobprogs(mob_rnum rnum)
{
  int i;

  if (rnum == NOWHERE) {/* assign the whole world */
	for (i = 0; i <= top_of_mobt; i++)
	  if (GET_PROG(&mob_proto[i]) != 0)
		ASSIGNMOB(mob_index[i].vnum, mobprogs[GET_PROG(&mob_proto[i])].function);
  } else  
	ASSIGNMOB(mob_index[rnum].vnum, mobprogs[GET_PROG(&mob_proto[rnum])].function);
}

/* assign special procedures to mobiles */
void assign_mobiles(void)
{
  ASSIGNMOB(1, puff);

  /* Immortal Zone */
  ASSIGNMOB(99, postmaster);

  assign_mobprogs(NOWHERE);
}



/* assign special procedures to objects */
void assign_objects(void)
{
  //ASSIGNOBJ(3034, bank);	/* atm */
  //ASSIGNOBJ(3036, bank);	/* cashcard */
}



/* assign special procedures to rooms */
void assign_rooms(void)
{
  room_rnum i;

  //ASSIGNROOM(3030, dump);
  //ASSIGNROOM(3031, pet_shops);

  if (CONFIG_DTS_ARE_DUMPS)
    for (i = 0; i <= top_of_world; i++)
      if (ROOM_FLAGGED(i, ROOM_DEATH))
	world[i].func = dump;
}
